==> sl-cia-2.0.243.tgz: package/lib/js_diff/source-map-handler.js <==
/*
    Written using "prototype", for easy migration to typeScript
 */
var consoleFileLogger = require("../sl-logger.js").consoleFileLogger;
var globalErrorHandler = require('../global-error-handler');
var utils = require('../utils');
var path = require("path");
var isValidPath = require('is-valid-path');

var WEBPACK_EMBEDDED_PREFIX = "webpack:/webpack/";
var REACT_GENERATED_FOLDER = "build/static/js";
var REACT_SOURCE_FOLDER = "src";
function SourceMapHandler(consumer, generatedName, generatedStart, generatedEnd, generatedFileName, logger) {
    if(!consumer){throw new Error("Source map consumer must be given")}
    if(!generatedName){throw new Error("generatedName must be given")}
    if(!generatedStart){throw new Error("generatedStart must be given")}
    if(!generatedEnd){throw new Error("generatedEnd must be given")}
    if(!generatedFileName){throw new Error("generatedFileName must be given")}
    this.consumer = consumer;
    this.start = this.resolveOriginalPosition(generatedStart, false);
    this.end = this.resolveOriginalPosition(generatedEnd, true);
    this.generatedFileName = generatedFileName;
    this.generatedName = generatedName;
    this.shouldSkipped = false;
    this.originalFilename;
    this.fixedOriginalFilename;
    this.logger = createLogger(logger);
    this.init()
}

SourceMapHandler.prototype.init = function () {
    if(!isValidPositionObject(this.start)){
        this.logger.debug("Invalid original start position, skipping this method");
        this.shouldSkipped = true;
        return;
    }
    this.originalFilename = this.start.source;
    var relativePath = convertToRelativePath(this.generatedFileName, this.originalFilename, this.logger);
    if(!relativePath){
        this.shouldSkipped = true;
        return;
    }
    if(isEmbeddedWebpackResource(relativePath, this.logger)){
        this.shouldSkipped = true;
        return;
    }
    var trimmedFileName = trimPackerPrefix(relativePath);
    if(shouldSkippedByPrefix(trimmedFileName)){
        this.logger.debug("File '%s' ignored due to it's prefix");
        this.shouldSkipped = true;
        return;
    }
    if(process.env.SL_IS_REACT){
        trimmedFileName = trimmedFileName.replace(REACT_GENERATED_FOLDER, REACT_SOURCE_FOLDER);
    }
    this.fixedOriginalFilename = trimmedFileName;

};

SourceMapHandler.prototype.resolveOriginalPosition = function (generatedPosition, isEnd) {
    var originalPosition = this.consumer.originalPositionFor({
        line: generatedPosition[0],
        column: isEnd ? generatedPosition[1] -1 : generatedPosition[1]
    });
    return new Position(originalPosition)
};


SourceMapHandler.prototype.getUniqueName = function () {
    return [this.getOriginalName(), this.fixedOriginalFilename, this.start.asFormatArray().join(',')].join('@')
};

SourceMapHandler.prototype.getOriginalName = function () {
    if(this.start && this.start.name){
        return this.start.name;
    }
    return this.generatedName;
};


/*
    sourceFileName is the file from source map (ie, '*.ts') file and originalFileName is the generated file (ie, "all.js").
 */
function convertToRelativePath(generatedFileName, originalFileName, logger) {
    if(originalFileName.toLowerCase().indexOf("node_modules") > -1){
        this.shouldSkipped = true;
        return;
    }
    var relativePath;
    try {
        var shortGeneratedFileName = removeCwd(generatedFileName);
        var shortOriginalFilename = removeCwd(originalFileName);
        if (shortOriginalFilename[0] == '/') {
            /* We have removed the cwd, so for example if:
            *  originalFilename: c:\nadav\my-project\file.js"
            *  filename: c:\nadav\my-project\src\file.ts"
            *  cwd: c:\nadav\my-project
            *  Then 'sourceFileName' will be "/src/file.ts" (due to the 'adjustPathSlashes()') and we want it to be relative "src/file.ts".
            */
            relativePath = shortOriginalFilename.substring(1);

        } else {
            /*
                If we removed the cwd and the file name does not start's with  a '/', we assumed that source path is relative to map file.
                and we also assumed that the map file is in the same directory as generated file.
                so we take the generated file directory and adds the source path to it, and then we normalize the path.
                so for example:
                 generated file: 'c:\projects\app\dist\bundle.js'
                 short generated file: 'dist\bundle.js'
                 map file: 'dist\bundle.js.map'
                 real path: 'c:\projects\app\src\foo.js'
                 source path: '..\src\foo.js'
                 combined path: 'dist\..\src\foo.js'
                 relative path: 'src\foo.js'
             */
            var generatedDir = path.dirname(shortGeneratedFileName);
            var combinedPath = path.join(generatedDir,shortOriginalFilename);
            var normalizedPath = path.normalize(combinedPath);
            relativePath = utils.adjustPathSlashes(normalizedPath);
        }
        logger.debug("Fixed relative path for file. Path after fixing: '" + relativePath  + "'.");
        return relativePath;
    } catch (err) {
        globalErrorHandler.setLastError(err, "Failed to fix real path for source maps. Error: ");
        return originalFileName;
    }
};

function isEmbeddedWebpackResource(fileName, logger) {
    if (fileName.indexOf(WEBPACK_EMBEDDED_PREFIX) === 0 ||
        fileName.indexOf("/" + WEBPACK_EMBEDDED_PREFIX) === 0) {
        logger.debug("Excluding fileName since it's an embedded resources. File:'" + fileName + "'.");
        return true; //Embedded resource that has no corresponding source
    }
    return false;
}

/**
 * This function will determine if the current file needs to be excluded.
 * The reason for excluding a file is if the files is loaded from an embedded resource.
 * For now, we consider a file as one that should be excluded if it contains any item like: " '/webpack:', '/webpack/', '~' ".
 */

function shouldSkippedByPrefix(fileName) {
    var generatedPatterns = new RegExp("/((\\bwebpack(?=:|\\/))|~)");
    return !isValidPath(fileName) || generatedPatterns.test(fileName);
}

function trimPackerPrefix(fileName) {
    fileName = utils.adjustPathSlashes(fileName);
    if (fileName.indexOf('webpack:///') === 0) {
        fileName = fileName.substring(11); //'webpack:///'.length
    }
    else if (fileName.indexOf("webpack:/") >= 0){
        var idx = fileName.indexOf("webpack:/");
        //Get just the filename (without the packer)
        fileName = fileName.substring(idx+9); // 9 == 'webpack:/'.length
    }
    return fileName;
}

function isValidPositionObject(position) {
    return position && position.source && position.line !== null && position.column !== null;
}

function removeCwd(filePath) {
    filePath = utils.adjustPathSlashes(filePath);
    var cwd = utils.adjustPathSlashes(process.cwd());
    return filePath.replace(cwd, "");
}

function createLogger(logger) {
    return logger ? logger.child({
        methodName: "SourceMapHandler"
    }) : consoleFileLogger("SourceMapHandler");
}

function Position(position){
    this.column = position.column;
    this.line = position.line;
    this.name = position.name;
    this.source = position.source;
    this.asFormatArray = function(){
        return [this.line, this.column];
    }
}

module.exports.SourceMapHandler = SourceMapHandler;

==> sl-cia-2.0.249.tgz: package/lib/js_diff/source-map-handler.js <==
/*
    Written using "prototype", for easy migration to typeScript
 */
var consoleFileLogger = require("../sl-logger.js").consoleFileLogger;
var globalErrorHandler = require('../global-error-handler');
var utils = require('../utils');
var path = require("path");
var isValidPath = require('is-valid-path');

var WEBPACK_EMBEDDED_PREFIX = "webpack:/webpack/";
var REACT_GENERATED_FOLDER = "build/static/js";
var REACT_SOURCE_FOLDER = "src";
function SourceMapHandler(consumer, generatedName, generatedStart, generatedEnd, generatedFileName, logger) {
    if(!consumer){throw new Error("Source map consumer must be given")}
    if(!generatedName){throw new Error("generatedName must be given")}
    if(!generatedStart){throw new Error("generatedStart must be given")}
    if(!generatedEnd){throw new Error("generatedEnd must be given")}
    if(!generatedFileName){throw new Error("generatedFileName must be given")}
    this.consumer = consumer;
    this.start = this.resolveOriginalPosition(generatedStart, false);
    this.end = this.resolveOriginalPosition(generatedEnd, true);
    this.generatedFileName = generatedFileName;
    this.generatedName = generatedName;
    this.shouldSkipped = false;
    this.originalFilename;
    this.fixedOriginalFilename;
    this.logger = createLogger(logger);
    this.init()
}

SourceMapHandler.prototype.init = function () {
    if(!isValidPositionObject(this.start)){
        this.logger.debug("Invalid original start position, skipping this method");
        this.shouldSkipped = true;
        return;
    }
    this.originalFilename = this.start.source;
    var relativePath = convertToRelativePath(this.generatedFileName, this.originalFilename, this.logger);
    if(!relativePath){
        this.shouldSkipped = true;
        return;
    }
    if(isEmbeddedWebpackResource(relativePath, this.logger)){
        this.shouldSkipped = true;
        return;
    }
    var trimmedFileName = trimPackerPrefix(relativePath);
    if(shouldSkippedByPrefix(trimmedFileName)){
        this.logger.debug("File '%s' ignored due to it's prefix", trimmedFileName);
        this.shouldSkipped = true;
        return;
    }
    if(process.env.SL_IS_REACT){
        trimmedFileName = trimmedFileName.replace(REACT_GENERATED_FOLDER, REACT_SOURCE_FOLDER);
    }
    this.fixedOriginalFilename = trimmedFileName;

};

SourceMapHandler.prototype.resolveOriginalPosition = function (generatedPosition, isEnd) {
    var originalPosition = this.consumer.originalPositionFor({
        line: generatedPosition[0],
        column: isEnd ? generatedPosition[1] -1 : generatedPosition[1]
    });
    return new Position(originalPosition)
};


SourceMapHandler.prototype.getUniqueName = function () {
    return [this.getOriginalName(), this.fixedOriginalFilename, this.start.asFormatArray().join(',')].join('@')
};

SourceMapHandler.prototype.getOriginalName = function () {
    if(this.start && this.start.name){
        return this.start.name;
    }
    return this.generatedName;
};


/*
    sourceFileName is the file from source map (ie, '*.ts') file and originalFileName is the generated file (ie, "all.js").
 */
function convertToRelativePath(generatedFileName, originalFileName, logger) {
    if(originalFileName.toLowerCase().indexOf("node_modules") > -1){
        this.shouldSkipped = true;
        return;
    }
    var relativePath;
    try {
        var shortGeneratedFileName = removeCwd(generatedFileName);
        var shortOriginalFilename = removeCwd(originalFileName);
        if (shortOriginalFilename[0] == '/') {
            /* We have removed the cwd, so for example if:
            *  originalFilename: c:\nadav\my-project\file.js"
            *  filename: c:\nadav\my-project\src\file.ts"
            *  cwd: c:\nadav\my-project
            *  Then 'sourceFileName' will be "/src/file.ts" (due to the 'adjustPathSlashes()') and we want it to be relative "src/file.ts".
            */
            relativePath = shortOriginalFilename.substring(1);

        } else {
            /*
                If we removed the cwd and the file name does not start's with  a '/', we assumed that source path is relative to map file.
                and we also assumed that the map file is in the same directory as generated file.
                so we take the generated file directory and adds the source path to it, and then we normalize the path.
                so for example:
                 generated file: 'c:\projects\app\dist\bundle.js'
                 short generated file: 'dist\bundle.js'
                 map file: 'dist\bundle.js.map'
                 real path: 'c:\projects\app\src\foo.js'
                 source path: '..\src\foo.js'
                 combined path: 'dist\..\src\foo.js'
                 relative path: 'src\foo.js'
             */
            var generatedDir = path.dirname(shortGeneratedFileName);
            var combinedPath = path.join(generatedDir,shortOriginalFilename);
            var normalizedPath = path.normalize(combinedPath);
            relativePath = utils.adjustPathSlashes(normalizedPath);
        }
        logger.debug("Fixed relative path for file. Path after fixing: '" + relativePath  + "'.");
        return relativePath;
    } catch (err) {
        globalErrorHandler.setLastError(err, "Failed to fix real path for source maps. Error: ");
        return originalFileName;
    }
};

function isEmbeddedWebpackResource(fileName, logger) {
    if (fileName.indexOf(WEBPACK_EMBEDDED_PREFIX) === 0 ||
        fileName.indexOf("/" + WEBPACK_EMBEDDED_PREFIX) === 0) {
        logger.debug("Excluding fileName since it's an embedded resources. File:'" + fileName + "'.");
        return true; //Embedded resource that has no corresponding source
    }
    return false;
}

/**
 * This function will determine if the current file needs to be excluded.
 * The reason for excluding a file is if the files is loaded from an embedded resource.
 * For now, we consider a file as one that should be excluded if it contains any item like: " '/webpack:', '/webpack/', '~' ".
 */

function shouldSkippedByPrefix(fileName) {
    var generatedPatterns = new RegExp("/((\\bwebpack(?=:|\\/))|~)");
    return !isValidPath(fileName) || generatedPatterns.test(fileName);
}

function trimPackerPrefix(fileName) {
    fileName = utils.adjustPathSlashes(fileName);
    if (fileName.indexOf('webpack:///') === 0) {
        fileName = fileName.substring(11); //'webpack:///'.length
    }
    else if (fileName.indexOf("webpack:/") >= 0){
        var idx = fileName.indexOf("webpack:/");
        //Get just the filename (without the packer)
        fileName = fileName.substring(idx+9); // 9 == 'webpack:/'.length
    }
    return fileName;
}

function isValidPositionObject(position) {
    return position && position.source && position.line !== null && position.column !== null;
}

function removeCwd(filePath) {
    filePath = utils.adjustPathSlashes(filePath);
    var cwd = utils.adjustPathSlashes(process.cwd());
    return filePath.replace(cwd, "");
}

function createLogger(logger) {
    return logger ? logger.child({
        methodName: "SourceMapHandler"
    }) : consoleFileLogger("SourceMapHandler");
}

function Position(position){
    this.column = position.column;
    this.line = position.line;
    this.name = position.name;
    this.source = position.source;
    this.asFormatArray = function(){
        return [this.line, this.column];
    }
}

module.exports.SourceMapHandler = SourceMapHandler;
